#- coding: utf-8
__Autor__ = "Luiz Fernando"
__Empresa__ = "Sig Informatica Ltda"
__Site__ = "http://www.siginformatica.com.br"
__Versao__ = "1.03"
__DataAlteracao__ = "08/04/2011"

import sys
import re
import requests
import json

if __name__ == '__main__':
    try:
        if sys.argv[1] == '-9':
            print(('cep.py\t\t\tVersao:' + __Versao__ + '\tData:'+ __DataAlteracao__) +
                  '\tValidador de CEPs')
        else:
            print(('\n\ncep.py - Validador de CEPs \n'
                   +'Versao: '+ __Versao__ + '\n'
                   +'Data: '+ __DataAlteracao__ + '\n'))
    except:
        print(('\n\ncep.py - Validador de CEPs \n'
               +'Versao: '+ __Versao__ + '\n'
               +'Data: '+ __DataAlteracao__ + '\n'))

class Cep(object):
    """
    Validador para informar se o CEP informado eh valido. Sao aceitos codigos
    de enderecamento postal em duas formas:Oito digitos consecutivos ou cinco
    digitos, hifen, tres digitos, ou seja, XXXXXXXX ou XXXXX-XXX, onde cada X
    pode ser qualquer digito entre 0 e 9.
    """

    def __init__(self, valor):
        self.cep = re.sub('[^0-9]', '', str(valor))
        if self.cep.__len__() == 5:
            self.cep = self.cep + '000'

    def validar(self):
        if self.cep.__len__() == 8:
            return True
        else:
            return False

    def getCepFormatado(self):
        return self.cep[:5]+"-"+ self.cep[5:]

    def getCepSemFormatar(self):
        return self.cep


# baixado de https://github.com/jbochi/cep
from BeautifulSoup import BeautifulSoup
import cookielib
import urllib
import urllib2
import HTMLParser

URL_CORREIOS = 'http://www.buscacep.correios.com.br/sistemas/buscacep/'

class Correios():
    def __init__(self, proxy=None):
        cj = cookielib.LWPCookieJar()
        cookie_handler = urllib2.HTTPCookieProcessor(cj)
        if proxy:
            proxy_handler = urllib2.ProxyHandler({'http': proxy})
            opener = urllib2.build_opener(proxy_handler, cookie_handler)
        else:
            opener = urllib2.build_opener(cookie_handler)
        urllib2.install_opener(opener)

    def _url_open(self, url, data=None, headers=None):
        if headers == None:
            headers = {}

        headers['User-agent'] = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'
        encoded_data = urllib.urlencode(data) if data else None
        url = URL_CORREIOS + url

        req = urllib2.Request(url, encoded_data, headers)
        handle = urllib2.urlopen(req)

        return handle

    def _parse_detalhe(self, html):
        soup = BeautifulSoup(html.decode('ISO-8859-1'))
        value_cells = soup.findAll('td', attrs={'class': 'value'})
        localidade, uf = values[2].text.split('/')
        values_dict = {
            'Logradouro': values[0].text,
            'Bairro': values[1].text,
            'Localidade': localidade,
            'UF': uf,
            'CEP': values[3].text
        }
        return values_dict

    def _parse_linha_tabela(self, tr):
        values = [cell for cell in tr.findAll('td')]
        values_dict = {}
        if values:
            localidade, uf = values[2].text.split('/')
            values_dict = {
                'logradouroDNEC': values[0].text,
                'bairro': values[1].text,
                'localidade': localidade,
                'uf': uf,
                'cep': values[3].text
            }
        return values_dict

    def _parse_tabela(self, html):
        soup = BeautifulSoup(html)
        linhas = soup.findAll('tr')
        lista = []
        for linha in linhas:
            res = self._parse_linha_tabela(linha)
            if res:
                lista.append(res)
        return lista


    def detalhe(self, posicao=0):
        """Retorna o detalhe de um CEP da última lista de resultados"""
        handle = self._url_open('detalheCEPAction.do', {'Metodo': 'detalhe',
                                                        'TipoCep': 2,
                                                        'Posicao': posicao + 1,
                                                        'CEP': None})
        html = handle.read()
        return self._parse_detalhe(html)

    def consulta(self, endereco, primeiro=False,
                 uf=None, localidade=None, tipo=None, numero=None):
        """Consulta site e retorna lista de resultados"""

        url = ('http://www.viacep.com.br/ws/%s/json' % endereco)
        req = requests.get(url)
        if req.status_code == 200:
            dados_json = json.loads(req.text)
            res = {
                'localidade': dados_json.get('localidade'),
                'bairro': dados_json.get('bairro'),
                'uf': dados_json.get('uf'),
                'logradouroDNEC': dados_json.get('logradouro'),
                'ibge': dados_json.get('ibge'),
            }
            return res

        if uf is None:
            url = 'resultadoBuscaCepEndereco.cfm'
            data = {
                'relaxation': endereco.encode('ISO-8859-1'),
                'tipoCEP': 'ALL',
                'semelhante': 'N',
                'cfm': 1,
                'Metodo': 'listaLogradouro',
                'TipoConsulta': 'relaxation',
                'StartRow': '1',
                'EndRow': '10'
            }
        else:
            url = 'consultaLogradouroAction.do'
            data = {
                'Logradouro': endereco.encode('ISO-8859-1'),
                'UF': uf,
                'TIPO': tipo,
                'Localidade': localidade.encode('ISO-8859-1'),
                'Numero': numero,
                'cfm': 1,
                'Metodo': 'listaLogradouro',
                'TipoConsulta': 'logradouro',
                'StartRow': '1',
                'EndRow': '10'
            }

        h = self._url_open(url, data)
        html = h.read()
        if primeiro:
            return self.detalhe()
        else:
            return self._parse_tabela(html)


"""
exemplo para acesso
>>> from cep import Correios
>>> c = Correios()
>>> resultados = c.consulta('Juarez')
>>> len(resultados)
100
>>> resultados[64]
{'Localidade': u'Jo\xe3o Pessoa', 'Bairro': u'Torre', 'UF': u'PB', 'Logradouro': u'Avenida Juarez T\xe1vora - de 1147/1148 a 1911/', 'CEP': u'58040-021'}
>>> c.detalhe(64)
{'Localidade': u'Jo\xe3o Pessoa', 'Bairro': u'Torre', 'UF': u'PB', 'Logradouro': u'Avenida Juarez T\xe1vora - de 1147/1148 a 1911/1912', 'CEP': u'58040-021'}
>>> c.consulta('91370000', primeiro=True)
{'Localidade': u'Porto Alegre', 'Bairro': u'Vila Ipiranga', 'UF': u'RS', 'Logradouro': u'Rua Alberto Silva - at\xe9 965/966', 'CEP': u'91370-000'}o
"""
